CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
136 lines
5.3 KiB
C++
136 lines
5.3 KiB
C++
/*! \file */
|
|
/* ************************************************************************
|
|
* Copyright (C) 2020-2025 Advanced Micro Devices, Inc. All rights Reserved.
|
|
*
|
|
* Permission is hereby granted, free of charge, to any person obtaining a copy
|
|
* of this software and associated documentation files (the "Software"), to deal
|
|
* in the Software without restriction, including without limitation the rights
|
|
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
|
|
* copies of the Software, and to permit persons to whom the Software is
|
|
* furnished to do so, subject to the following conditions:
|
|
*
|
|
* The above copyright notice and this permission notice shall be included in
|
|
* all copies or substantial portions of the Software.
|
|
*
|
|
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
|
|
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
|
|
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
|
|
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
|
|
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
|
|
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
|
|
* THE SOFTWARE.
|
|
*
|
|
* ************************************************************************ */
|
|
|
|
#include "rocsparse/rocsparse.h"
|
|
#include <hip/hip_runtime_api.h>
|
|
#include <iostream>
|
|
|
|
#define HIP_CHECK(stat) \
|
|
{ \
|
|
if(stat != hipSuccess) \
|
|
{ \
|
|
std::cerr << "Error: hip error in line " << __LINE__ << std::endl; \
|
|
return -1; \
|
|
} \
|
|
}
|
|
|
|
#define ROCSPARSE_CHECK(stat) \
|
|
{ \
|
|
if(stat != rocsparse_status_success) \
|
|
{ \
|
|
std::cerr << "Error: rocsparse error in line " << __LINE__ << std::endl; \
|
|
return -1; \
|
|
} \
|
|
}
|
|
|
|
int main(int argc, char* argv[])
|
|
{
|
|
// Query device
|
|
int ndev;
|
|
HIP_CHECK(hipGetDeviceCount(&ndev));
|
|
|
|
if(ndev < 1)
|
|
{
|
|
std::cerr << "No HIP device found" << std::endl;
|
|
return -1;
|
|
}
|
|
|
|
// Query device properties
|
|
hipDeviceProp_t prop;
|
|
HIP_CHECK(hipGetDeviceProperties(&prop, 0));
|
|
|
|
std::cout << "Device: " << prop.name << std::endl;
|
|
|
|
// rocSPARSE handle
|
|
rocsparse_handle handle;
|
|
ROCSPARSE_CHECK(rocsparse_create_handle(&handle));
|
|
|
|
// Print rocSPARSE version and revision
|
|
int ver;
|
|
char rev[64];
|
|
|
|
ROCSPARSE_CHECK(rocsparse_get_version(handle, &ver));
|
|
ROCSPARSE_CHECK(rocsparse_get_git_rev(handle, rev));
|
|
|
|
std::cout << "rocSPARSE version: " << ver / 100000 << "." << ver / 100 % 1000 << "."
|
|
<< ver % 100 << "-" << rev << std::endl;
|
|
|
|
// Input data
|
|
|
|
// Number of non-zeros of the sparse vector
|
|
rocsparse_int nnz = 3;
|
|
|
|
// Sparse index vector
|
|
rocsparse_int hx_ind[3] = {0, 3, 5};
|
|
|
|
// Sparse value vector
|
|
rocsparse_float_complex hx_val[3] = {{1.0, 4.0}, {2.0, 5.0}, {3.0, 6.0}};
|
|
|
|
// Dense vector
|
|
rocsparse_float_complex hy[9] = {{1.0, -9.0},
|
|
{2.0, -8.0},
|
|
{3.0, -7.0},
|
|
{4.0, -6.0},
|
|
{5.0, -5.0},
|
|
{6.0, -4.0},
|
|
{7.0, -3.0},
|
|
{8.0, -2.0},
|
|
{9.0, -1.0}};
|
|
|
|
// Index base
|
|
rocsparse_index_base idx_base = rocsparse_index_base_zero;
|
|
|
|
// Offload data to device
|
|
rocsparse_int* dx_ind;
|
|
rocsparse_float_complex* dx_val;
|
|
rocsparse_float_complex* dy;
|
|
|
|
HIP_CHECK(hipMalloc((void**)&dx_ind, sizeof(rocsparse_int) * nnz));
|
|
HIP_CHECK(hipMalloc((void**)&dx_val, sizeof(rocsparse_float_complex) * nnz));
|
|
HIP_CHECK(hipMalloc((void**)&dy, sizeof(rocsparse_float_complex) * 9));
|
|
|
|
HIP_CHECK(hipMemcpy(dx_ind, hx_ind, sizeof(rocsparse_int) * nnz, hipMemcpyHostToDevice));
|
|
HIP_CHECK(
|
|
hipMemcpy(dx_val, hx_val, sizeof(rocsparse_float_complex) * nnz, hipMemcpyHostToDevice));
|
|
HIP_CHECK(hipMemcpy(dy, hy, sizeof(rocsparse_float_complex) * 9, hipMemcpyHostToDevice));
|
|
|
|
// Call cdotci to compute the dot product of a complex conjugate sparse vector
|
|
// with a dense vector
|
|
rocsparse_float_complex dot;
|
|
ROCSPARSE_CHECK(rocsparse_cdotci(handle, nnz, dx_val, dx_ind, dy, &dot, idx_base));
|
|
|
|
// Print result
|
|
std::cout << "Dot product: " << dot << std::endl;
|
|
|
|
// Clear rocSPARSE
|
|
ROCSPARSE_CHECK(rocsparse_destroy_handle(handle));
|
|
|
|
// Clear device memory
|
|
HIP_CHECK(hipFree(dx_ind));
|
|
HIP_CHECK(hipFree(dx_val));
|
|
HIP_CHECK(hipFree(dy));
|
|
|
|
return 0;
|
|
}
|